use std::path::Path;
use std::process::Command;

pub fn pull(repo: &str) -> Option<String> {
    let repo_path = Path::new(repo);

    add_git_safe_dir(repo_path.to_str().unwrap());

    if repo_path.exists() {
        let _ = Command::new("git")
            .arg("pull")
            .current_dir(repo_path)
            .status()
            .expect("Failed to pull latest changes");
    } else {
        let _ = Command::new("git")
            .args(["clone", repo, repo])
            .status()
            .expect("Failed to clone repository");
    }
    get_commit_hash(repo)
}

pub fn get_commit_hash(repo: &str) -> Option<String> {
    let output = Command::new("git")
        .args(["rev-parse", "HEAD"])
        .current_dir(repo)
        .output()
        .ok()?;
    Some(String::from_utf8_lossy(&output.stdout).trim().to_string())
}

pub fn is_git(dir: &str) -> bool {
    std::fs::exists(std::path::Path::new(dir).join(".git")).unwrap()
}

pub fn add_git_safe_dir(dir: &str) {
    let abs_path = std::path::absolute(std::path::Path::new(dir)).unwrap();
    let path = abs_path.to_str().unwrap();
    log::info!("Adding {path} to git safe dirs");

    let output = Command::new("git")
        .args(["config", "--global", "--add", "safe.directory", path])
        .output()
        .expect("Failed to execute git config");

    if !output.status.success() {
        eprintln!(
            "Error setting safe.directory: {}",
            String::from_utf8_lossy(&output.stderr)
        );
    }
}
