use argh::FromArgs;
use based::get_pg;
use comrade::service::ServiceManager;
use comrade::{defer, serde_json};
use owl::{prelude::*, query, save, set_global_db, update};
use package::{IndexEntry, rebuild_pkgs, reindex_pkg};
use serde::{Deserialize, Serialize};
use yansi::{Color, Paint};
use std::process::Command;

pub mod builder;
pub mod git;
pub mod package;

pub struct PackageIndex;

impl PackageIndex {
    pub fn init() {
        let db = Database::filesystem("./owl_db");
        set_global_db!(db);
    }

    pub fn is_present(&self, repo: &str, pkg: &str) -> bool {
        !query!(|x: &IndexEntry| x.repo == repo && x.pkg == pkg).is_empty()
    }

    pub fn insert_if_not_present(&mut self, repo: &str, pkg: &str) {
        if !self.is_present(repo, pkg) {
            let entry = IndexEntry {
                repo: repo.to_string(),
                pkg: pkg.to_string(),
                last_commit: None,
                last_pkgver: None,
                id: Id::String(format!("{repo}-{pkg}")),
            };
            save!(entry);
        }
    }

    pub fn set_last(
        &mut self,
        commit: Option<String>,
        pkgver: Option<String>,
        repo: &str,
        pkg: &str,
    ) {
        println!("setting last");
        let mut entry = query!(|x: &IndexEntry| x.repo == repo && x.pkg == pkg);
        println!("have entry {}", entry.len());
        update!(&mut entry, |entry| {
            entry.last_commit = commit.clone();
            entry.last_pkgver = pkgver.clone();
        });
    }
}

pub fn list_pkgs() {
    let pkgs: Vec<Model<IndexEntry>> = query!(|_| true);

    for pkg in pkgs {
        let pkg = pkg.read();
        if pkg.last_commit.is_none() && pkg.last_pkgver.is_none() {
            println!("{}", format!("- {} / {} @ NONE", pkg.repo, pkg.pkg).paint(Color::Red));
        } else {
            println!(
                "- {} / {} @ {} [{}]",
                pkg.repo, pkg.pkg, pkg.last_pkgver.as_ref().unwrap(), pkg.last_commit.as_ref().unwrap()
            );
        }
    }
}

#[derive(FromArgs)]
/// pacco makepkg
pub struct PaccoMakePkgArgs {
    #[argh(subcommand)]
    pub cmd: PaccoMakePkgCmds,
}

#[derive(FromArgs)]
#[argh(subcommand)]
pub enum PaccoMakePkgCmds {
    List(ListCommand),
    Build(BuildCommand),
}

#[derive(FromArgs)]
#[argh(subcommand, name = "list")]
/// list packages
pub struct ListCommand {}

#[derive(FromArgs)]
#[argh(subcommand, name = "build")]
/// build packages
pub struct BuildCommand {}

#[tokio::main]
async fn main() {
    env_logger::init();
    PackageIndex::init();

    let args: PaccoMakePkgArgs = argh::from_env();

    match args.cmd {
        PaccoMakePkgCmds::List(_) => list_pkgs(),
        PaccoMakePkgCmds::Build(_) => {
            reindex_pkg(&std::path::Path::new("./repositories")).await;

            // TODO : summary ui

            rebuild_pkgs().await;

            log::info!("Writing index");

            let s = ServiceManager::new();
            let s = s.spawn();
            defer!(move || s.join().unwrap());
        }
    }
}
